﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using Infragistics;

namespace WpfControls
{
	public class ChartDataItem
	{
		public string Text { get; set; }
		public string Value { get; set; }
		public string Typ { get; set; }

		public string TooltipText { get; set; }

		public Visibility MarkerVisibility
		{
			get
			{
				if (this.Value == null)
					return Visibility.Hidden;

				return (this.Value == "0") ? Visibility.Hidden : Visibility.Visible;
			}
		}

		public ChartDataItem()
		{
			this.Text = String.Empty;
			this.Value = String.Empty;
			this.Typ = String.Empty;

			this.TooltipText = string.Empty;
		}

		public ChartDataItem(DataRow aDataRow, string aCategoryColumnName = "text", string aValueColumnName = "value", string aKeyColumnName = "typ", bool aUseShortName = false)
		{
			this.Text = aDataRow[aCategoryColumnName] as string ?? string.Empty;

			this.Value = aDataRow[aValueColumnName] as string ?? string.Empty;
			this.Typ = aDataRow.Table.Columns.Contains(aKeyColumnName)
				? aDataRow[aKeyColumnName].ToString() : string.Empty;

			this.TooltipText = aDataRow[aCategoryColumnName] as string ?? this.Text;
		}
	}

	public class ChartsViewModel : ViewModelBase
	{

		public GroupBy StackedBarData { get; set; }

		public IEnumerable ItemsSource
		{
			get { return this.DataItems; }
		}

		private IEnumerable<ChartDataItem> iDataItems;

		public IEnumerable<ChartDataItem> DataItems
		{
			get
			{
				if (this.IsStackedBarChart)
					return this.StackedBarData.ItemsSource.Cast<ChartDataItem>();

				return this.iDataItems;
			}
			set
			{
				if (this.IsStackedBarChart)
					this.StackedBarData.ItemsSource = value;
				else
					this.iDataItems = value;
				this.NotifyPropertyChanged();
			}
		}

		private string iHeaderText;

		public string HeaderText
		{
			get { return this.iHeaderText ?? string.Empty; }
			set
			{
				this.iHeaderText = value;
				this.NotifyPropertyChanged();
				this.FirePropertyChanged("HeaderTextVisibility");
			}
		}

		private string iFooterText;

		public string FooterText
		{
			get { return this.iFooterText ?? String.Empty; }
			set
			{
				this.iFooterText = value;
				this.NotifyPropertyChanged();
				this.FirePropertyChanged("FooterTextVisibility");
			}
		}

		public Visibility FooterTextVisibility
		{
			get { return (string.IsNullOrEmpty(this.FooterText)) ? Visibility.Collapsed : Visibility.Visible; }
		}

		public Visibility HeaderTextVisibility
		{
			get { return (string.IsNullOrEmpty(this.HeaderText)) ? Visibility.Collapsed : Visibility.Visible; }
		}

		public Visibility LegendVisibility { get; set; }

		public Visibility StackedBarChartVisibility
		{
			get { return this.IsStackedBarChart ? Visibility.Visible : Visibility.Hidden; }
		}

		public Visibility BarChartVisibility
		{
			get { return this.IsStackedBarChart ? Visibility.Hidden : Visibility.Visible; }
		}

		private bool iIsStackedBarChart;

		public bool IsStackedBarChart
		{
			get { return this.iIsStackedBarChart; }
			set
			{
				this.iIsStackedBarChart = value;
				this.FirePropertyChanged("StackedBarChartVisibility");
				this.FirePropertyChanged("BarChartVisibility");
			}
		}

		private double iMaxValue = double.NaN;

		public double MaxValue
		{
			get
			{
				var mVariableExtent = 50.0;

				if (double.IsNaN(this.iMaxValue))
				{
					if (this.DataItems.Any())
						this.iMaxValue = this.DataItems
						                     .Select(item => double.Parse(item.Value))
						                     .Max();
					else
						this.iMaxValue = 0.0;
				}


				if (this.iMaxValue < 10)
					mVariableExtent = 5.0;
				else if (this.iMaxValue < 50)
					mVariableExtent = 10.0;
				else if (this.iMaxValue < 200)
					mVariableExtent = 20.0;


				return (this.iMaxValue + mVariableExtent);
			}
		}

		/// <summary>
		/// Dient dazu, dass bei einem MaxValue von 10 immer in Einerschritten und nicht mit 0.5er angezeigt wird.
		/// Sobald der Wert größer wird NaN geliefert, das bewirkt, dass die Intervallanpassung automatisch geschieht.
		/// </summary>
		public double Interval
		{
			get { return this.MaxValue < 10.0 ? 1 : double.NaN; }
		}

		public ChartsViewModel()
		{
			this.IsStackedBarChart = false;
			this.DataItems = new ObservableCollection<ChartDataItem>();
		}

		public ChartsViewModel(DataTable aDataTable, bool aStackedChart, string aCategoryColumnName = "text",
		                       string aValuesColumnName = "value", string aGroupColumnName = "typ", bool aUseShortName = false)
		{
			this.IsStackedBarChart = aStackedChart;
			var mGroupBy = new GroupBy
				{
					KeyMemberPath = "Typ",
					ValueMemberPath = "Value",
					GroupMemberPath = "Text"
				};

			var mDataItems = new ObservableCollection<ChartDataItem>();
			if (aDataTable != null)
			{
				mDataItems = new ObservableCollection<ChartDataItem>(aDataTable
					                        .AsEnumerable()
					                        .Select(
						                        mRow =>
						                        new ChartDataItem(mRow, aCategoryColumnName,
						                                        aValuesColumnName, aGroupColumnName))
					                        .ToList());
			}

			if (IsStackedBarChart)
			{
				mGroupBy.ItemsSource = mDataItems;
				this.StackedBarData = mGroupBy;
			}
			else
			{
				this.DataItems = mDataItems;
			}

		}
	}
}
